@@ -5,10 +5,13 @@ import pygtk
 pygtk.require('2.0')
 import pango
 import gtk
+from glib import GError
 import gobject
 import time
 from gameclock.game import Game
 import gameclock.clock
+from gameclock import __version__, __license__, __copyright__
+from pkg_resources import resource_filename, Requirement
 
 class GameclockUI:
     """this class handles most of the UI and turned-based logic
@@ -214,15 +217,14 @@ class GameclockUI:
             controls.show()
             window.vbox.pack_start(controls)
 
-            # this may be a reset, init from old values
-            try:
-                miliseconds = self.game.miliseconds
-                default_clock = self.game.first_clock.__class__.__name__
-            except AttributeError:
-                self.ui.maybe_print("exception")
+            if self.ui.game is None:
                 miliseconds = Game.default_time
                 default_clock = Game.clock_type
-
+            else:
+                # this is a reset, init from old values
+                miliseconds = self.ui.game.default_time
+                default_clock = getattr(gameclock.clock, self.ui.game.first_clock.__class__.__name__)
+ 
             left_controls = gtk.VBox(False, 0)
             left_controls.show()
 
@@ -356,12 +358,11 @@ much right right side if it *is* present.
 
         def about_dialog(self, action = None):
             dialog = gtk.AboutDialog()
-            dialog.set_version(version)
-            dialog.set_copyright(copyright)
-            #dialog.set_license("license?")
-            dialog.set_comments("A simple game clock to be used for Chess or any board game")
+            dialog.set_version(__version__)
+            dialog.set_copyright(__copyright__)
+            dialog.set_license(__license__)
+            dialog.set_comments("A simple game clock to be used for Chess or any board game. Written by Antoine Beaupré <anarcat@koumbit.org>")
             dialog.set_website("https://redmine.koumbit.net/projects/gameclock")
-            dialog.set_authors(["Antoine Beaupré <anarcat@anarcat.ath.cx>"])
             dialog.connect("response", lambda a, b: dialog.destroy())
             dialog.show()
 
@@ -371,6 +372,19 @@ much right right side if it *is* present.
     def main(self):
         """create the main user interface with GTK"""
 
+        try:
+            icon = resource_filename(Requirement.parse("gameclock"), 'gameclock.svg')
+            gtk.window_set_default_icon_from_file(icon)
+        except GError as e:
+            self.maybe_print("could not find icon: %s" % e)
+            try:
+                # this is necessary because, for some reason, the code
+                # above doesn't look in the data_files defined in setup.py
+                gtk.window_set_default_icon_from_file("/usr/share/pixmaps/gameclock.svg")
+            except GError as e:
+                self.maybe_print("could not find icon: %s" % e)
+                pass
+
         # create a new window
         self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
 
@@ -399,6 +413,7 @@ much right right side if it *is* present.
         # turn counter
         self.turns = gtk.Label()
         self.turns.modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color('white'))
+        self.turns.show()
         vlayout.pack_start(self.turns, False, False, 0)
 
         # the clocks
@@ -406,6 +421,14 @@ much right right side if it *is* present.
         self.clock_table.show()
         vlayout.pack_start(self.clock_table, True, True, 0)
 
+        self.status_bar = gtk.Statusbar()
+        vlayout.pack_start(self.status_bar, False, False, 0)
+        self.status_bar.set_has_resize_grip(False);
+        # make sure the text is visible
+        label = self.status_bar.get_message_area().get_children()[0]
+        label.modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color('white'))
+        self.status_bar.show()
+
         vlayout.show()
 
         if self.fullscreen:
@@ -464,6 +487,11 @@ much right right side if it *is* present.
         if not self.cur_clock:
             self.cur_clock = self.first_clock
 
+    def status(self, text, priority = 'info'):
+        context_id = self.status_bar.get_context_id(priority)
+        self.maybe_print(text)
+        self.status_bar.push(context_id, text)
+
     def maybe_print(self, text):
         if self.verbose:
             t = ""
@@ -497,6 +525,7 @@ much right right side if it *is* present.
         this handler is ran periodically through a timeout signal to make sure that the current clock is updated
         """
 
+        # in hourglass, we just update both clocks all the time
         if isinstance(self.game.first_clock, gameclock.clock.HourglassClock):
             p = self.game.first_clock
             while p:
@@ -538,16 +567,19 @@ much right right side if it *is* present.
 
     def handle_key_press(self, widgets, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
+        # see this FAQ for more information about keycodes: http://faq.pygtk.org/index.py?file=faq05.005.htp&req=edit
         # notice how we do not handle the arrow keys/home/end/scrlock and num pad
-        if ( keyname == 'Shift_L' or keyname == 'Caps_Lock'
+        if event.state & gtk.gdk.CONTROL_MASK or event.state & gtk.gdk.MOD1_MASK:
+            # gtk.gdk.SHIFT_MASK is okay
+            self.maybe_print("key pressed with mod/control ignored");
+        elif ( keyname == 'Shift_L' or keyname == 'Caps_Lock'
           or keyname == 'Alt_L' or keyname == 'Super_L'
           or event.hardware_keycode == 49 # ~
           or event.hardware_keycode in range(52, 56) # z-b
           or event.hardware_keycode in range(38, 42) # a-g
           or event.hardware_keycode in range(23, 28) # tab-q-t
           or event.hardware_keycode in range(10, 15) # 1-6
-          or event.hardware_keycode in range(67, 70) # F1-F4
-          or keyname == 'Escape'):
+          or event.hardware_keycode in range(67, 70)): # F1-F4
             if self.game.cur_clock == self.game.first_clock:
                 self.end_turn()
         elif ( keyname == 'Shift_R' or keyname == 'Return'
@@ -565,6 +597,9 @@ much right right side if it *is* present.
             self.handle_pause()
         elif keyname == 'space':
             self.end_turn()
+        elif keyname == 'Escape':
+            if self.fullscreen:
+                self.handle_fullscreen()
         self.maybe_print("key %s (%d/%d) was pressed" % (keyname, event.keyval, event.hardware_keycode))
 
     def handle_fullscreen(self, action = None):
@@ -599,24 +634,36 @@ much right right side if it *is* present.
         # some reason it doesn't work to just update the old clock label, we need to update both
         self.refresh()
         self.hilight()
-        self.maybe_print("ended turn")
+        self.maybe_print("turn finished")
         # mouse pointer debugging code, see #4511
         if event and event.device:
-            self.maybe_print("widget: %s" % widget)
             self.maybe_print("device name: %s" % event.device.name)
-            self.maybe_print("device idt: %s" % event.deviceid)
+            self.maybe_print("device: %s" % event.device)
+            try:
+                if self.prev_dev == event.device:
+                    self.maybe_print("devices are the same")
+                else:
+                    self.maybe_print("devices are diffrent")
+            except AttributeError:
+                pass
+            self.maybe_print("devices: %s" % gtk.gdk.display_get_default().list_devices())
+            self.prev_dev = event.device.get_source()
 
     def handle_pause(self, action = None):
         """pause handler
         
         just a stub for the game engine for now
         """
-        self.maybe_print("pausing game")
-        self.game.pause()
+        if self.game.pause():
+            self.status("game paused")
+        else:
+            self.status("game resumed")
+            # the timeout handler is removed when we pause, resume
+            self.timeout_source = gobject.timeout_add(self.loop_timeout, self.refresh_current)
 
     def start_game(self):
         self.game.start()
         self.timeout_source = gobject.timeout_add(self.loop_timeout, self.refresh_current)
         self.turns.set_label("Turn %d" % self.game.get_turns())
-        self.turns.show()
-        self.maybe_print("starting game, refresh rate %dms" % self.loop_timeout)
+        self.maybe_print("refresh rate %dms" % self.loop_timeout)
+        self.status("game started")
